/*
 *  Copyright 2020-2023 Google LLC
 *  Copyright 2020-2023 EPAM Systems, Inc
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.opengroup.osdu.notification.repo;

import java.util.Collections;
import java.util.HashSet;
import java.util.Optional;
import java.util.Set;
import java.util.concurrent.TimeUnit;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.notification.config.RedisProperties;
import org.springframework.data.redis.core.Cursor;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.core.ScanOptions;
import org.springframework.stereotype.Service;

@Slf4j
@Service
@RequiredArgsConstructor
public class SubscriptionCacheRepo {

  public static final String POISON = ":poison";
  private final RedisProperties redisProperties;
  private final RedisTemplate<String, Subscription> redisTemplate;

  public void writeSubscription(String dataPartitionId, Subscription subscription) {
    String subscriberCacheKey = getSubscriberCacheKey(
        dataPartitionId,
        subscription.getTopic(),
        subscription.getNotificationId()
    );
    redisTemplate.opsForValue().set(
        subscriberCacheKey,
        subscription
    );
  }

  public Subscription readSubscription(String dataPartitionId, String topic,
      String subscriptionId) {
    String subscriberCacheKey = getSubscriberCacheKey(
        dataPartitionId,
        topic,
        subscriptionId
    );
    return redisTemplate.opsForValue().get(subscriberCacheKey);
  }

  /**
   * To avoid Register service spamming, if a subscriber was deleted, but replicated events are
   * still being processed.
   *
   * @param dataPartitionId
   * @param topic
   * @param subscriptionId
   */
  public void poisonSub(String dataPartitionId, String topic, String subscriptionId) {
    String subPoisonKey = getSubscriberCacheKey(dataPartitionId, topic, subscriptionId) + POISON;

    redisTemplate.opsForValue().set(
        subPoisonKey,
        new Subscription(),
        redisProperties.getRedisExpiration(),
        TimeUnit.SECONDS
    );
  }

  /**
   * Used to check if subscribers were not found in Register.
   *
   * @param dataPartitionId
   * @param topic
   * @param subscriptionId
   * @return empty Subscription
   */
  public Subscription checkIfSubIsPoisoned(String dataPartitionId, String topic,
      String subscriptionId) {
    String subPoisonKey = getSubscriberCacheKey(dataPartitionId, topic, subscriptionId) + POISON;
    return redisTemplate.opsForValue().get(subPoisonKey);
  }

  public boolean deleteSubscription(String dataPartitionId, String topic, String subscriptionId) {
    String subscriberCacheKey = getSubscriberCacheKey(dataPartitionId, topic, subscriptionId);
    Boolean delete = redisTemplate.delete(subscriberCacheKey);
    return Boolean.TRUE.equals(delete);
  }

  public Set<Subscription> getDataPartitionSubscribers(String dataPartitionId) {
    return getSubscriptions(dataPartitionId + ":*");
  }

  public Set<Subscription> getTopicSubscribers(String dataPartitionId, String topic) {
    return getSubscriptions(dataPartitionId + ":" + topic + ":*");
  }

  private HashSet<Subscription> getSubscriptions(String pattern) {
    ScanOptions options = ScanOptions.scanOptions()
        .match(pattern)
        .build();

    HashSet<String> keys = new HashSet<>();

    try (Cursor<String> cursor = redisTemplate.scan(options)) {
      while (cursor.hasNext()) {
        keys.add(cursor.next());
      }
    }

    return new HashSet<>(Optional.ofNullable(redisTemplate.opsForValue().multiGet(keys))
        .orElse(Collections.emptyList()));
  }

  private static String getSubscriberCacheKey(String dataPartitionId, String topic, String subId) {
    return dataPartitionId + ":" + topic + ":" + subId;
  }
}
