/*
 *  Copyright 2020-2023 Google LLC
 *  Copyright 2020-2023 EPAM Systems, Inc
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.opengroup.osdu.notification.service;

import java.util.List;
import java.util.Objects;
import java.util.Set;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.notification.repo.SubscriptionCacheRepo;
import org.springframework.stereotype.Component;

@Component
@RequiredArgsConstructor
@Slf4j
public class ExternalSubscriptionsManager {

  private final SubscriptionCacheRepo subscriptionCacheRepo;
  private final RegisterSubscriptionService subscriptionService;

  public void invokeTenantSubscribers(String dataPartitionId) {
    Set<Subscription> members = subscriptionCacheRepo.getDataPartitionSubscribers(dataPartitionId);
    if (Objects.isNull(members) || members.isEmpty()) {
      log.debug("Subscription info cache wasn't found for tenant {}", dataPartitionId);
      reloadSubscriptionInfoCache(dataPartitionId);
    }
  }

  public Set<Subscription> getSubscriptionsForTopic(String dataPartitionId, String topicName) {
    return subscriptionCacheRepo.getTopicSubscribers(dataPartitionId, topicName);
  }

  public Subscription getSubscription(String dataPartitionId, String topic, String subscriptionId) {
    Subscription subscription = subscriptionCacheRepo.readSubscription(dataPartitionId, topic, subscriptionId);
    if (subscription != null) {
      return subscription;
    }

    log.debug("Subscription: {} not in cache, checking if it available in Register.", subscriptionId);
    subscription = subscriptionCacheRepo.checkIfSubIsPoisoned(dataPartitionId, topic, subscriptionId);
    if (subscription != null) {
      log.debug("Subscription:{} marked as not available in cache.", subscriptionId);
      return null;
    }
    return retrieveSubscriptionFromHandler(dataPartitionId, subscriptionId, topic);
  }

  private Subscription retrieveSubscriptionFromHandler(String dataPartitionId, String subscriptionId, String topic) {
    Subscription subscription = subscriptionService.getSubscriptionsByNotificationId(dataPartitionId,subscriptionId);
    if (subscription == null) {
      log.debug("Subscription:{} not available in Register. Marking it as not available in cache.", subscriptionId);
      subscriptionCacheRepo.poisonSub(dataPartitionId, topic, subscriptionId);
      return null;
    }
    subscriptionCacheRepo.writeSubscription(dataPartitionId, subscription);
    return subscription;
  }

  public boolean removeSubscription(String dataPartitionId, String topic, String subscriptionId) {
    return subscriptionCacheRepo.deleteSubscription(dataPartitionId, topic, subscriptionId);
  }

  public void updateSubscription(String dataPartitionId, String subscriptionId) {
    Subscription subscription = subscriptionService.getSubscriptionsByNotificationId(
        dataPartitionId,
        subscriptionId
    );
    if(Objects.isNull(subscription)){
      log.warn("Cannot update sub config with id:{} in the cache. Skipping it.", subscriptionId);
    }else {
      subscriptionCacheRepo.writeSubscription(dataPartitionId, subscription);
    }
  }

  private void reloadSubscriptionInfoCache(String dataPartitionId) {
    List<Subscription> fragmentarySubInfos = subscriptionService.getAllSubscriptionInfos(dataPartitionId);
    for (Subscription freagmentedSubscription : fragmentarySubInfos) {
      Subscription subscription = subscriptionService.getSubscriptionsByNotificationId(dataPartitionId, freagmentedSubscription.getNotificationId());
      if(Objects.isNull(subscription)){
        log.warn("Cannot init cache for sub id:{}. Empty response from Register. Skipping it.", freagmentedSubscription.getNotificationId());
      }else {
        subscriptionCacheRepo.writeSubscription(dataPartitionId, subscription);
      }
    }
  }
}