/*
 *  Copyright 2020-2023 Google LLC
 *  Copyright 2020-2023 EPAM Systems, Inc
 *
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 */

package org.opengroup.osdu.notification.pubsub;

import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.times;
import static org.mockito.Mockito.verify;

import java.util.HashMap;
import java.util.List;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.junit.MockitoJUnitRunner;
import org.opengroup.osdu.core.common.model.notification.Subscription;
import org.opengroup.osdu.oqm.core.model.OqmAckReplier;
import org.opengroup.osdu.oqm.core.model.OqmMessage;
import org.opengroup.osdu.notification.service.ExternalSubscriptionsManager;
import org.opengroup.osdu.notification.repo.SubscriptionCacheRepo;
import org.opengroup.osdu.notification.pubsub.receiver.OqmConfigurationEventReceiver;

@RunWith(MockitoJUnitRunner.class)
public class OqmConfigurationEventReceiverTest {

    @InjectMocks
    private OqmConfigurationEventReceiver sut;
    @Mock
    private OqmAckReplier replier;
    @Mock
    private Subscription subscription;
    @Mock
    private List<Subscription> subscriptionInfos;
    @Mock
    private ExternalSubscriptionsManager externalSubscriptionsManager;
    @Mock
    private SubscriptionCacheRepo subscriptionCacheRepo;

    @Test
    public void testReceiveMessageSubscriptionCreated() {
        OqmMessage createMessage = OqmMessage.builder().data("Subscription Created").attributes(new HashMap<>()).build();
        sut.receiveMessage(createMessage, replier);
        verify(externalSubscriptionsManager, times(1)).updateSubscription(any(), any());
        verify(replier, times(1)).ack();
    }

    @Test
    public void testReceiveMessageSubscriptionUpdated() {
        OqmMessage updatedMessage = OqmMessage.builder().data("Subscription Updated").attributes(new HashMap<>()).build();
        sut.receiveMessage(updatedMessage, replier);
        verify(externalSubscriptionsManager, times(1)).updateSubscription(any(), any());
        verify(replier, times(1)).ack();
    }

    @Test
    public void testReceiveMessageSubscriptionDeleted() {
        String partitionId = "test";
        String topic = "topic1";
        String subId = "4";

        OqmMessage deletedMessage = OqmMessage.builder()
                .data("Subscription Deleted")
                .attributes(new HashMap<>() {{
                    put("subscription-id", subId);
                    put("topic", topic);
                    put("data-partition-id", partitionId);
                }})
                .build();
        sut.receiveMessage(deletedMessage, replier);
        verify(externalSubscriptionsManager, times(1)).removeSubscription(partitionId, topic, subId);
        verify(replier, times(1)).ack();
    }

    @Test
    public void testReceiveMessageUnsupported() {
        OqmMessage unsupportedMessage = OqmMessage.builder().data("Unsupported").attributes(new HashMap<>()).build();
        sut.receiveMessage(unsupportedMessage, replier);
        verify(replier, times(1)).ack();
    }
}