/* Copyright Amazon.com, Inc. or its affiliates. All Rights Reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.example;

import com.amazonaws.services.sqs.AmazonSQS;
import com.amazonaws.services.sqs.AmazonSQSClientBuilder;
import com.amazonaws.services.sqs.model.SendMessageResult;
import com.google.common.hash.Hashing;
import org.apache.http.HttpStatus;
import org.opengroup.osdu.core.common.model.http.AppException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.context.annotation.RequestScope;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.servlet.HandlerMapping;

import jakarta.servlet.http.HttpServletRequest;
import javax.validation.constraints.NotBlank;
import java.io.IOException;
import java.nio.charset.StandardCharsets;
import java.util.Base64;
import java.util.Map;
import java.util.Scanner;

@RestController
@RequestMapping("/")
@RequestScope
@Validated
public class PushEndpointController {
    private final AmazonSQS sqsClient;
    private final int maxQueueNum;
    private static final Logger LOGGER = LoggerFactory.getLogger(PushEndpointController.class);

    public PushEndpointController() {
        sqsClient = AmazonSQSClientBuilder.defaultClient();
        maxQueueNum = Integer.parseInt(System.getenv("MAX_QUEUE_NUM"));
    }

    private static String verifySubscriptionForChallenge(String crc) {
		String response = OSDUSecretString.getSecret() + crc;
        response = Hashing.sha256()
                          .hashString(response, StandardCharsets.UTF_8)
                          .toString();
        response = Base64.getEncoder().encodeToString(response.getBytes());
        return response;
    }

    @GetMapping("/{count}")
    @PreAuthorize("@hmacVerification.verifyHmac()")
    public ResponseEntity<VerifySubscriptionResponse> verifySubscription(@RequestParam("crc") @NotBlank String crc, HttpServletRequest request) {
        getQueueNumberFromRequest(request);
        String response = verifySubscriptionForChallenge(crc);
        VerifySubscriptionResponse returnValue = new VerifySubscriptionResponse(response);
        LOGGER.info(returnValue.toString());
        return ResponseEntity.ok(returnValue);
    }

    private String readMessage(HttpServletRequest request) throws IOException {
        Scanner scan = new Scanner(request.getInputStream());
        StringBuilder builder = new StringBuilder();
        while (scan.hasNextLine()) {
            builder.append(scan.nextLine());
        }
        return builder.toString();
    }

    private void throwAppException(String message) {
		throw new AppException(HttpStatus.SC_BAD_REQUEST, "Bad Request", message);
    }

    private int getQueueNumberFromRequest(HttpServletRequest request) {
        Object uriTemplateObject = request.getAttribute(HandlerMapping.URI_TEMPLATE_VARIABLES_ATTRIBUTE);
        if (uriTemplateObject == null || !(uriTemplateObject instanceof Map)) {
            throwAppException("Invalid request path!");
        }
        Map<String, String> uriTempalteVariables = (Map<String, String>) request.getAttribute(HandlerMapping.URI_TEMPLATE_VARIABLES_ATTRIBUTE);
        String queueNumber = uriTempalteVariables.get("count");
        int returnValue = Integer.parseInt(queueNumber);
        if (returnValue >= maxQueueNum) {
            throwAppException("Invalid queue number!");
        }
        return returnValue;
    }

    private String getQueueURL(HttpServletRequest request) {
        String queueURL = System.getenv("QUEUE_NUM_" + getQueueNumberFromRequest(request));
        if (queueURL == null) {
            throwAppException("Queue not found!");
        }
        return queueURL;
    }

    @PostMapping("/{count}")
    @PreAuthorize("@hmacVerification.verifyHmac()")
    public ResponseEntity<SendMessageResult> pushNotificationToQueue(HttpServletRequest request) throws IOException {
        String queueURL = getQueueURL(request);
        String message = readMessage(request);
        SendMessageResult returnValue = sqsClient.sendMessage(queueURL, message);
        LOGGER.info(returnValue.toString());
        return ResponseEntity.ok(returnValue);
    }
}

record VerifySubscriptionResponse(String responseHash) {

}
